/*
    Scanbot Barcode Scanner SDK Flutter Plugin

    Copyright (c) 2020 doo GmbH. All rights reserved.
    https://scanbot.io/sdk
 */
package io.scanbot.barcode.sdk.flutter

import io.flutter.plugin.common.MethodCall
import io.scanbot.sdk.barcode.entity.BarcodeDensity
import io.scanbot.sdk.barcode.entity.EngineMode
import io.scanbot.sdk.ui.view.barcode.batch.configuration.BatchBarcodeScannerConfiguration
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeImageGenerationType
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeScannerConfiguration
import org.json.JSONObject

fun readBarcodeScannerConfiguration(call: MethodCall): BarcodeScannerConfiguration {
    val arguments = call.arguments as Json
    val configuration: BarcodeScannerConfiguration =
        BarcodeScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())

    if (call.hasArgument("engineMode")) {
        configuration.setEngineMode(engineModeFromJson(arguments["engineMode"] as String))
    }
    if (call.hasArgument("enableCameraButtonTitle")) {
        configuration.setEnableCameraButtonTitle(arguments["enableCameraButtonTitle"] as String)
    }
    if (call.hasArgument("enableCameraExplanationText")) {
        configuration.setEnableCameraExplanationText(arguments["enableCameraExplanationText"] as String)
    }
    if (call.hasArgument("barcodeImageGenerationType")) {
        configuration.setBarcodeImageGenerationType(BarcodeImageGenerationType.valueOf(arguments["barcodeImageGenerationType"] as String))
    }
    if (call.hasArgument("minFocusDistanceLock")) {
        configuration.setMinFocusDistanceLock(arguments["minFocusDistanceLock"] as Boolean)
    }
    return configuration
}

fun readBatchBarcodeScannerConfiguration(call: MethodCall): BatchBarcodeScannerConfiguration {
    val arguments = call.arguments as Json
    val configuration: BatchBarcodeScannerConfiguration =
        BatchBarcodeScannerConfiguration.fromJson(JSONObject(arguments as Map<*, *>).toString())
    if (call.hasArgument("engineMode")) {
        configuration.setEngineMode(engineModeFromJson(arguments["engineMode"] as String))
    }
    if (call.hasArgument("enableCameraButtonTitle")) {
        configuration.setEnableCameraButtonTitle(arguments["enableCameraButtonTitle"] as String)
    }
    if (call.hasArgument("enableCameraExplanationText")) {
        configuration.setEnableCameraExplanationText(arguments["enableCameraExplanationText"] as String)
    }
    if (call.hasArgument("minFocusDistanceLock")) {
        configuration.setMinFocusDistanceLock(arguments["minFocusDistanceLock"] as Boolean)
    }
    return configuration
}

/** Extract the barcode density value as enum. */
internal fun extractBarcodeDensityFrom(string: String?) : BarcodeDensity? {
    when(string ?: "") {
        "LOW" -> return BarcodeDensity.LOW
        "HIGH" -> return BarcodeDensity.HIGH
    }
    return null
}

/** Extract the barcode engine mode value as enum. */
private fun engineModeFromJson(engine: String): EngineMode {
    return if (engine == "Legacy") EngineMode.LegacyFast
    else EngineMode.NextGen
}
