//
//  BarcodeScannerConfiguration.swift
//  barcode_scanner
//
//  Created by Seifeddine Bouzid on 15.05.23.
//

import Foundation
import ScanbotBarcodeScannerSDK

struct BarcodeScannerConfiguration {
    
    var flashEnabled: Bool?
    var zoomConfiguration: SBSDKBaseScannerZoomConfiguration?
    var viewFinderConfiguration: SBSDKBaseScannerViewFinderConfiguration?
    var scannerConfiguration: ScannerConfiguration?
    
    init() { }
    
    init(_ arguments: [String: Any?]?) {
        if let arguments {
            self = BarcodeScannerConfiguration.parseArguments(arguments)
        }
    }
    
    private static func parseArguments(_ arguments: [String: Any?]) -> BarcodeScannerConfiguration {
        var configuration = BarcodeScannerConfiguration()
        
        if let flashEnabled = arguments["flashEnabled"] as? Bool {
            configuration.flashEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = arguments["cameraZoomFactor"] as? CGFloat {
            let zoomConfig = SBSDKBaseScannerZoomConfiguration.default()
            zoomConfig.initialZoomFactor = cameraZoomFactor
            configuration.zoomConfiguration = zoomConfig
        }
        
        configuration.viewFinderConfiguration = arguments.finderConfiguration
        
        if let scannerConfiguration: [String: Any] = arguments["scannerConfiguration"] as? [String: Any] {
            configuration.scannerConfiguration = ScannerConfiguration(scannerConfiguration)
        }
        return configuration
    }
    
    struct ScannerConfiguration {
        
        var acceptedDocumentTypes: [SBSDKBarcodeDocumentType]?
        var acceptedBarcodeTypes: [SBSDKBarcodeType]?
        var engineMode: SBSDKBarcodeEngineMode?
        var additionalDetectionParameters: SBSDKBarcodeAdditionalParameters?
        var barcodeImageGenerationType: SBSDKBarcodeImageGenerationType?
        
        init() { }
        
        init(_ arguments: [String: Any]) {
            print(arguments)
            self = ScannerConfiguration.parseArguments(arguments: arguments)
        }
        
        private static func parseArguments(arguments: [String: Any]) -> ScannerConfiguration {
            var configuration = ScannerConfiguration()
            
            if arguments["acceptedDocumentFormats"] is [String] {
                configuration.acceptedDocumentTypes = SBSDKBarcodeDocumentType.fromArguments(arguments)
            }
            
            if arguments["barcodeFormats"] is [String] {
                configuration.acceptedBarcodeTypes = SBSDKBarcodeType.fromArguments(arguments)
            }
            
            if arguments["engineMode"] is String {
                configuration.engineMode = SBSDKBarcodeEngineMode.fromArguments(arguments)
            }
            
            if let additionalParams: [String: Any] = arguments["additionalParameters"] as? [String: Any] {
                configuration.additionalDetectionParameters = SBSDKBarcodeAdditionalParameters(from: additionalParams)
            }
            
            if let barcodeImageGenerationType = arguments["barcodeImageGenerationType"] as? String {
                configuration.barcodeImageGenerationType = SBSDKBarcodeImageGenerationType(from: barcodeImageGenerationType)
            }
            return configuration
        }
    }
}
