//
//  Scanbot Barcode Scanner SDK Flutter Plugin
//
//  Copyright (c) 2020 doo GmbH. All rights reserved.
//  https://scanbot.io/sdk
//

import UIKit
import ScanbotBarcodeScannerSDK
import Flutter

class BarcodeScannerPresenter: NSObject {
    private var resultHandler: FlutterResult?
    private weak var presenter: UIViewController?
    private var shouldWaitForImage = false
    private var barcodeResults: [SBSDKBarcodeScannerResult]?
    
    func present(on presenter: UIViewController,
                 with configuration: SBSDKUIBarcodeScannerConfiguration,
                 result: @escaping FlutterResult) {
        self.resultHandler = result
        self.presenter = presenter
        self.shouldWaitForImage = configuration.behaviorConfiguration.barcodeImageGenerationType != .none
        
        let controller = SBSDKUIBarcodeScannerViewController.createNew(with: configuration, andDelegate: self)
        controller.modalPresentationStyle = .fullScreen
        presenter.present(controller, animated: true, completion: nil)
    }
    
    private func processResult(barcodeResults: [SBSDKBarcodeScannerResult], imageURL: URL? = nil) {
        self.presenter?.dismiss(animated: true) { [weak self] in 
            let result: [String: Any?] = ["barcodeImageURI": imageURL?.absoluteString,
                                          "barcodeItems": barcodeResults.map({$0.encodedDictionary})]
            self?.resultHandler?(result.encodedString)
            self?.resultHandler = nil
            self?.presenter = nil
            self?.barcodeResults = nil
        }
    }
    
    private func clean() {
        self.resultHandler?("CANCELED")
        self.resultHandler = nil
        self.presenter = nil
        self.barcodeResults = nil
    }
}

extension BarcodeScannerPresenter: SBSDKUIBarcodeScannerViewControllerDelegate {
    func qrBarcodeDetectionViewController(_ viewController: SBSDKUIBarcodeScannerViewController,
                                          didDetect barcodeResults: [SBSDKBarcodeScannerResult]) {
        
        if self.shouldWaitForImage {
            if barcodeResults.isEmpty {
                return
            }
            self.barcodeResults = barcodeResults
            if let image = barcodeResults[0].sourceImage {
                viewController.isRecognitionEnabled = false
                if let path = StorageHandler.default.addImage(image) {
                    guard let url = URL(string: path) else { return }
                    self.processResult(barcodeResults: barcodeResults, imageURL: url)
                    self.shouldWaitForImage = false
                } else {
                    viewController.isRecognitionEnabled = true
                }
            }
        } else {
            self.processResult(barcodeResults: barcodeResults)
        }
    }
    
    func qrBarcodeDetectionViewControllerDidCancel(_ viewController: SBSDKUIBarcodeScannerViewController) {
        self.clean()
    }
    
    func qrBarcodeDetectionViewControllerDidTimeout(_ viewController: SBSDKUIBarcodeScannerViewController) {
        self.clean()
    }
}
