//
//  BatchScannerPresenter.swift
//  barcode_scanner
//
//  Created by Yevgeniy Knizhnik on 20.01.21.
//

import Foundation
import ScanbotBarcodeScannerSDK
import Flutter

class BatchScannerPresenter: NSObject {
    private var resultHandler: FlutterResult?
    private weak var presenter: UIViewController?
    private var dataMapper: SBSDKUIBarcodesBatchScannerMapping?
    
    func present(on presenter: UIViewController,
                 with configuration: SBSDKUIBarcodesBatchScannerConfiguration,
                 dataMapper: SBSDKUIBarcodesBatchScannerMapping?,
                 result: @escaping FlutterResult) {
        
        self.resultHandler = result
        self.presenter = presenter
        
        let controller = SBSDKUIBarcodesBatchScannerViewController.createNew(with: configuration, andDelegate: self)
        self.dataMapper = dataMapper
        controller.modalPresentationStyle = .fullScreen
        presenter.present(controller, animated: true) { [weak controller, weak dataMapper] in
            controller?.dataMapper = dataMapper
        }
    }
    
    private func processResult(barcodeResults: [SBSDKBarcodeScannerResult]) {
        self.presenter?.dismiss(animated: true) { [weak self] in
            let result: [String: Any?] = ["barcodeItems": barcodeResults.map({$0.encodedDictionary})]
            self?.resultHandler?(result.encodedString)
            self?.clean()
        }
    }
    
    private func clean() {
        self.resultHandler = nil
        self.presenter = nil
        self.dataMapper = nil
    }
}

extension BatchScannerPresenter: SBSDKUIBarcodesBatchScannerViewControllerDelegate {
    func barcodesBatchScannerViewController(_ viewController: SBSDKUIBarcodesBatchScannerViewController,
                                            didFinishWith barcodeResults: [SBSDKUIBarcodeMappedResult]) {
        viewController.dataMapper?.loadMappedData(forBarcode: barcodeResults.first!.barcode, completion: nil)
        
        self.processResult(barcodeResults: barcodeResults.map({ $0.barcode }))
    }
    
    func barcodesBatchScannerViewControllerDidCancel(_ viewController: SBSDKUIBarcodesBatchScannerViewController) {
        self.clean()
    }
}
