import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:logging/logging.dart';

import 'classical_camera.dart';

abstract class LiveDetector {
  @visibleForTesting
  static late String channelName;

  late MethodChannel detectorChannel;

  @protected
  bool enabled = true;

  Map<String, dynamic> arguments = {};

  void registerResultHandler();

  attach(ScanbotCameraController controller) {
    try {
      channelName = "scanbot_sdk_camera.detector_${controller.viewId}";
      detectorChannel = MethodChannel(channelName);
      registerResultHandler();
    } catch (e) {
      Logger.root.severe(e);
    }
  }

  Future<void> pauseDetection() async {
    try {
      enabled = false;
      await detectorChannel.invokeMethod("disable");
    } catch (e) {
      Logger.root.severe(e);
    }
    return;
  }

  Future<void> resumeDetection() async {
    try {
      enabled = true;
      await detectorChannel.invokeMethod("enable");
    } catch (e) {
      Logger.root.severe(e);
    }
    return;
  }
}
